﻿using System;
using System.Collections.Generic;
using System.Net.Http;
using System.Security.Claims;
using System.Text;
using System.Threading.Tasks;
using Microsoft.AspNetCore.Authentication;
using Microsoft.AspNetCore.Authentication.Cookies;
using Microsoft.AspNetCore.Authorization;
using Microsoft.AspNetCore.Http;
using Microsoft.AspNetCore.Mvc;
using Microsoft.Extensions.Configuration;
using Newtonsoft.Json;
using NLog;
using backstage.Helpers;
using backstage.Models;
using backstage.Models.Users;
using Newtonsoft.Json.Linq;

namespace backstage.Controllers
{
    public class UserController : Controller
    {
        private readonly IHttpContextAccessor _httpContextAccessor;
        private readonly IConfiguration _config;
        private readonly ICallApi _callApi;

        private static Logger logger = LogManager.GetCurrentClassLogger();
        /// <summary>
        /// 讀取組態用
        /// </summary>

        public UserController(IConfiguration config, ICallApi callApi, IHttpContextAccessor httpContextAccessor)
        {
            _config = config;
            _callApi = callApi;
            _httpContextAccessor = httpContextAccessor;
        }

        [Authorize]
        [HttpGet]
        public IActionResult Register()
        {
            return View();
        }


        [HttpGet]
        public async Task<IActionResult> Login()
        {
            if (HttpContext.User.Identity.IsAuthenticated)
            {
                return RedirectToAction("Index", "Home");
            }
            return View();

        }


        [HttpGet]
        public async Task<IActionResult> ListUsers()
        {
            var UserResponse = new UserResponse();
            var url = _config["IP"]+ "/admin/list";
            // 確認使用者是否已經登入
            if (!User.Identity.IsAuthenticated)
            {
                return RedirectToAction("Login");


            }



            var httpMethod = HttpMethod.Post;
            // 取得使用者的 "token" Claim 值
            string token = User.FindFirstValue("token");

            var types = new[] { "all" };
            var data = new { inc = types };

            var parameters = new Dictionary<string, string>
              {
                    { "token", token },
                    { "types", JsonConvert.SerializeObject(data)}
                    //{ "types", "{\"inc\":[\"all\"]}"}

                };

            var apiResult = await _callApi.CallAPI(url, parameters, httpMethod);
            if (apiResult.IsSuccess)
            {
                UserResponse = JsonConvert.DeserializeObject<UserResponse>(apiResult.Data.ToString());
                if (UserResponse.r == 0)
                {
                    return View(UserResponse);
                }

            }

            return Ok(apiResult.Data.ToString());




        }


        [HttpGet]
        public async Task<IActionResult> ListDepartments()
        {
            var DepartmentsResponse = new DepartmentsResponse();
            var url = _config["IP"] + "/merchant/list";
            // 確認使用者是否已經登入
            if (!User.Identity.IsAuthenticated)
            {
                return RedirectToAction("Login");


            }


            var httpMethod = HttpMethod.Post;
            // 取得使用者的 "token" Claim 值
            string token = User.FindFirstValue("token");

        
            var parameters = new Dictionary<string, string>
              {
                    { "token", token }

                };

            var apiResult = await _callApi.CallAPI(url, parameters, httpMethod);
            if (apiResult.IsSuccess)
            {
                DepartmentsResponse = JsonConvert.DeserializeObject<DepartmentsResponse>(apiResult.Data.ToString());
                if (DepartmentsResponse.r == 0)
                {
                    return View(DepartmentsResponse);
                }

            }

            return Ok(apiResult.Data.ToString());


        }


        [HttpPost]
        [ValidateAntiForgeryToken]
        public async Task<IActionResult> Login(User user, string returnUrl)
        {

            if (!ModelState.IsValid)
            {
                return View(user);
            }



            //var u = await _UserRepository.Login(user);
            var u = new UserLogin();



            var url = _config["IP"];
            var parameters = new Dictionary<string, string>
              {
                    { "username", user.username },
                    { "pwd", user.pwd },
                    { "reqPermFlag[0]","admin-login"}
              };
            var httpMethod = HttpMethod.Post;

            try
            {
                var apiResult = await _callApi.CallAPI(url, parameters, httpMethod);

                if (apiResult.IsSuccess)
                {
                    // API 呼叫成功，進行相應的處理

                    u = JsonConvert.DeserializeObject<UserLogin>(apiResult.Data.ToString());
                    if (u.r == 0 && !string.IsNullOrEmpty(u.token))
                    {
                        #region 寫入TOKEN
                        int LoginExpireMinute = Convert.ToInt32(_config["LoginExpireMinute"]);
                        string key = _config["TokenKey"];

                        var Cookieoptions = new CookieOptions();
                        ////時間一到就會把Token從Cookie洗掉,如果Expires沒比API的Expires長,則每次Token過期都須重新登入
                        Cookieoptions.Expires = DateTime.Now.AddMinutes(LoginExpireMinute);
                        //Cookieoptions.SameSite = SameSiteMode.Strict;
                        Cookieoptions.HttpOnly = true;

                        var userClaims = new ClaimsIdentity(new[]
                                {
                       new Claim("username", user.username),
                        new Claim("token", u.token),
                         }, CookieAuthenticationDefaults.AuthenticationScheme);

                        //將 ClaimsIdentity 設定給 ClaimsPrincipal (持有者) 
                        ClaimsPrincipal principal = new ClaimsPrincipal(userClaims);
                        //登入動作

                        try
                        {
                            await HttpContext.SignInAsync(CookieAuthenticationDefaults.AuthenticationScheme, principal, new AuthenticationProperties()
                            {
                                //是否可以被刷新
                                AllowRefresh = true,
                                // 設置了一個 1 天 有效期的持久化 cookie
                                IsPersistent = true, //IsPersistent = false，瀏覽器關閉即刻登出
                                                     //用戶頁面停留太久，逾期時間，在此設定的話會覆蓋Startup.cs裡的逾期設定
                                ExpiresUtc = DateTime.Now.AddMinutes(LoginExpireMinute),


                            });
                        }
                        catch (Exception ex)
                        {
                            logger.Trace(ex.Message + ex.InnerException?.Message);
                        }
                        #endregion


                        return RedirectToAction("Index", "Home");
                    }
                    else
                    {
                        // API 呼叫失敗，進行相應的錯誤處理
                        return BadRequest(apiResult.Message);
                    }

                }
                else
                {
                    // API 呼叫失敗，進行相應的錯誤處理
                    return BadRequest(apiResult.Message);
                }
            }
            catch (Exception ex)
            {
                // 處理 API 呼叫發生的例外狀況
                return StatusCode(500, $"API 呼叫發生錯誤：{ex.Message}");
            }





        }


        [HttpGet]
        public async Task<IActionResult> Logout()
        {

            await HttpContext.SignOutAsync();
            foreach (var cookieKey in Request.Cookies.Keys)
                HttpContext.Response.Cookies.Delete(cookieKey);
            return RedirectToAction(nameof(Login));

        }


        public async Task<IActionResult> List()
        {


            //TempData["IsSuccess"] = result.IsSuccess;
            //TempData["msg"] = result.Message;
            //return View(viewModel);
            throw new NotImplementedException();
        }
        public async Task<ResultModel> Delete(int id)
        {
            throw new NotImplementedException();
        }

    }
}